# frozen_string_literal: true

require 'rails_helper'

RSpec.describe(Operation, type: :model) do
  it 'has a valid factory' do
    expect(true).to be_truthy
  end

  describe 'code' do
    let!(:operation) { create(:operation, code: 'test') }

    it 'should upcase the code' do
      expect(operation.reload.code).to eq('TEST')
    end

    it 'should be unique and case insensitive' do
      expect(build(:operation, code: 'test')).to_not be_valid
      expect(build(:operation, code: 'TEST')).to_not be_valid
    end
  end

  describe 'updated_tagset_operation_title' do
    let!(:operation) { create(:operation, :with_tagset_root) }
    let!(:tagset) { create(:tagset, operation:) }

    it 'should update the title of the root tagset' do
      operation.update(title: 'new title')
      expect(operation.root_tagset.reload.title).to eq('Projet new title')
    end

    it 'should not update the operation_title of all tagset' do
      operation.update(title: 'new title')
      expect(tagset.reload.operation_title).to eq('new title')
    end
  end

  describe 'update_directories' do
    let!(:operation) { create(:operation) }
    let!(:tagset_directory) { create(:tagset_directory, operation:) }
    let!(:tagset_doc) { create(:tagset_doc, operation:) }

    RSpec.shared_examples 'update directories attributes' do |attribute, type|
      it "#{attribute} with the right values" do
        old_value = tagset_doc.send(attribute)
        new_value = case type.to_sym
                    when :date
                      Date.today - 5.days
                    when :user
                      create(:user, first_name: 'Jean', last_name: 'Dupont').id
                    end

        operation.update(attribute => new_value)
        expect(tagset_directory.reload.send(attribute)).to eq(new_value)
        expect(tagset_doc.reload.send(attribute)).to eq(old_value)
      end
    end

    include_examples 'update directories attributes', :start_date, :date
    include_examples 'update directories attributes', :end_date, :date
    include_examples 'update directories attributes', :responsible, :user
    include_examples 'update directories attributes', :current_user_id, :user
  end

  describe 'destroy_tagsets' do
    let!(:operation) { create(:operation) }
    let!(:tagset_doc) { create(:tagset_doc, :with_file, operation:) }
    let!(:tagset_directory) { create(:tagset_directory, operation:) }

    it 'should destroy all tagsets' do
      operation.destroy
      expect(Tagset.where(operation_id: operation.id)).to be_empty
    end
  end

  describe 'generate_operation_tree' do
    let!(:doc_csv) { create(:doc, title: 'doc.csv', file_path: 'packages/spirit_hub/spec/fixtures/operation_trees/sample1.csv') }
    let!(:operation) { create(:operation, title: 'Operation 1', classement_plan_id: doc_csv.id) }

    it 'should create all record' do
      subject
      expect(operation.root_tagset.title).to eq('Projet Operation 1')
      terrain = Tagset.find_by(operation:, title: 'Terrain')
      expect(Tagset.where(:id.in => terrain.tagset_ids).pluck(:title)).to match_array(['Descriptif du foncier', 'Acquisition Propriétaire XX', 'Etudes réalisées'])
      expect(SpiritHub::Completeness.find_by(tagset_id: terrain.id).title).to eq('Convention(s) collectivités')
    end
  end
end
