# frozen_string_literal: true

require 'rails_helper'

RSpec.describe(SpiritHub::DirectoryCompletenessUpdater, type: :service) do
  before do
    SpiritHub::HubdocCycle.import_cycle_from_json
  end

  subject { described_class.call(tagset) }

  let(:tagset) { create(:tagset_directory) }

  let(:create_completeness) do
    proc { |tagset, title| create(:completeness, title:, tagset:, min_objects: 1) }
  end

  let(:create_child_directory) do
    proc do |tagset, *traits, **opts|
      child = create(:tagset_directory, *traits, **opts)

      tagset.update!(tagset_ids: tagset.tagset_ids.to_a + [child.id])

      child
    end
  end

  RSpec.shared_examples 'directory is marked as complete' do
    it 'sets the content_completeness to 100' do
      subject
      expect(tagset.reload.content_completeness).to eq 100
    end

    it 'sets the current_state to complete' do
      subject
      expect(tagset.reload.current_state).to eq described_class::STATE_COMPLETE
    end
  end

  RSpec.shared_examples 'directory is marked as incomplete' do |content_completeness:|
    it 'sets the content_completeness to the correct percentage' do
      subject
      expect(tagset.reload.content_completeness).to eq content_completeness
    end

    it 'sets the current_state to incomplete' do
      subject
      expect(tagset.reload.current_state).to eq described_class::STATE_INCOMPLETE
    end
  end

  context 'when passing a non directory tagset' do
    let(:tagset) { create(:tagset_doc) }

    it 'raises an error' do
      expect { subject }.to raise_error ArgumentError
    end
  end

  context 'when directory has no associated completeness rule' do
    before do
      tagset.completeness.each(&:destroy!)
      tagset.reload
    end

    context 'with no child directories' do
      before do
        tagset.children_tagsets.each(&:destroy!)
        tagset.reload
      end

      include_examples 'directory is marked as complete'
    end

    context 'with child directories all complete' do
      let!(:child1) { create_child_directory.call(tagset, :complete) }
      let!(:child2) { create_child_directory.call(tagset, :complete) }

      include_examples 'directory is marked as complete'
    end

    context 'with child directories and some incomplete' do
      let!(:child1) { create_child_directory.call(tagset, :complete) }
      let!(:child2) { create_child_directory.call(tagset, :incomplete, content_completeness: 70) }
      let!(:child3) { create_child_directory.call(tagset, :incomplete, content_completeness: 10) }

      # due to the incomplete completeness rule
      # content_completeness: 100% (child1) + 70% (child2) + 10% (child3) / 3
      include_examples 'directory is marked as incomplete', content_completeness: 60
    end
  end

  context 'when directory has associated completeness rules' do
    let!(:completeness1) { create_completeness.call(tagset, 'Completeness 1') }
    let!(:completeness2) { create_completeness.call(tagset, 'Completeness 2') }

    context 'and all of them are completed' do
      before do
        completeness1.update!(completed: true)
        completeness2.update!(completed: true)
      end

      context 'with no child directories' do
        before do
          tagset.children_tagsets.each(&:destroy!)
          tagset.reload
        end

        include_examples 'directory is marked as complete'
      end

      context 'with child directories all complete' do
        let!(:child1) { create_child_directory.call(tagset, :complete) }
        let!(:child2) { create_child_directory.call(tagset, :complete) }

        include_examples 'directory is marked as complete'
      end

      context 'with child directories and one incomplete' do
        let!(:child1) { create_child_directory.call(tagset, :complete) }
        let!(:child2) { create_child_directory.call(tagset, :incomplete, content_completeness: 20) }

        # due to the incomplete completeness rule
        # content_completeness: 100% (completeness1) + 100% (completeness2) + 100% (child1) + 20% (child2) / 4
        include_examples 'directory is marked as incomplete', content_completeness: 80
      end
    end

    context 'when one of them is incomplete' do
      before do
        completeness1.update!(completed: true)
        completeness2.update!(completed: false)
      end

      context 'with no child directories' do
        before do
          tagset.children_tagsets.each(&:destroy!)
          tagset.reload
        end

        # due to the incomplete completeness rule
        # content_completeness: 100% (completness1) + 0% (completeness2) / 2
        include_examples 'directory is marked as incomplete', content_completeness: 50
      end

      context 'with child directories all complete' do
        let!(:child1) { create_child_directory.call(tagset, :complete) }
        let!(:child2) { create_child_directory.call(tagset, :complete) }

        # due to the incomplete completeness rule
        # content_completeness: 100% (completness1) + 0% (completeness2) + 100% (child1) + 100% (child2) / 4
        include_examples 'directory is marked as incomplete', content_completeness: 75
      end

      context 'with child directories and one incomplete' do
        let!(:child1) { create_child_directory.call(tagset, :complete) }
        let!(:child2) { create_child_directory.call(tagset, :incomplete, content_completeness: 40) }

        # due to the incomplete completeness rule
        # content_completeness: 100% (completness1) + 0% (completeness2) + 100% (child1) + 40% (child2) / 4
        include_examples 'directory is marked as incomplete', content_completeness: 60
      end
    end
  end

  context 'when passing the opt to update all the directory parents recursively' do
    subject { described_class.call(tagset, { update_parents: true }) }

    let(:parent1) { create(:tagset_directory, :incomplete) }
    let(:parent2) { create_child_directory.call(parent1, :incomplete) }
    let(:tagset) { create_child_directory.call(parent2, :complete) }

    it 'updates the completeness from the directory and from all its parents recursively' do
      expect(parent1).to_not be_completed
      expect(parent1.content_completeness).not_to eq 100
      expect(parent2).to_not be_completed
      expect(parent2.content_completeness).not_to eq 100

      subject

      expect(parent1.reload).to be_completed
      expect(parent1.content_completeness).to eq 100
      expect(parent2.reload).to be_completed
      expect(parent2.content_completeness).to eq 100
      expect(tagset.reload).to be_completed
      expect(tagset.content_completeness).to eq 100
    end
  end
end
